import com.niggle.data.*;
import com.niggle.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.util.List;


/**
 * The code that provides the dispatching
 * logic for our first pass at a rolodex
 */
public class MiniRoloServletInteraction extends ServletInteraction
{

    /**
     * This constructor must have exactly this signature, since
     * Niggle invokes it by reflective lookup.
     */
    public MiniRoloServletInteraction(HttpServletRequest req, HttpServletResponse resp, NiggleConfig config)
    throws IOException
    {
        super(req, resp, config);
    }

    /**
     * Displays our default view, which is simply
     * a view of all the entries.
     */
    public void execDefault() throws IOException
    {
        displayEntries();
    }

    /**
     * deletes the record indicated by the unique_id
     * parameter and redirects to the default view.
     */
    public void execDelete() throws IOException
    {
        Integer recordID = new Integer(getParameter("unique_id"));
        getData().delete(recordID);
        displayEntries();
    }

    /**
     * displays a page that allows
     * the user to edit the fields of a rolodex entry.
     */
    public void execEdit() throws IOException
    {
        page = getPage("edit.html");
        page.expose("entry", getEntry());
    }

    /**
     * Processes the form embedded in the edit page template
     * and redirects to the default view.
     */
    public void execProcess() throws IOException
    {
        Record entry = getEntry();
        if (entry.isImmutable())
        {
            entry = entry.getMutableCopy();
            DataUtil.fillRecordFromServletRequest(entry, this.request);
            getData().update(entry);
        } else
        {
            DataUtil.fillRecordFromServletRequest(entry, this.request);
            getData().insert(entry);
        }
        displayEntries();
    }

    protected void exposeDefaultVariables() throws IOException
    {
        super.exposeDefaultVariables();
        page.expose("defaultURL", getURL("default"));
        page.expose("editURL", getURL("edit"));
        page.expose("deleteURL", getURL("delete"));
        page.expose("processURL", getURL("process"));
    }

    /**
     * Displays our default view, which is simply
     * a view of all the entries.
     */
    private void displayEntries() throws IOException
    {
        page = getPage("entries.html");
        MutableDataSource data = getData();
        List entries = data.select(null);
        page.expose("entries", entries);
    }



    /**
     * returns a handle to our one and only data source.
     */
    private MutableDataSource getData() throws IOException
    {
        DataRegistry reg = DataRegistryImpl.getDefaultRegistry();
        return (MutableDataSource) reg.getDataSource("rolodex_data");
    }

    /**
     * returns the rolodex entry based on the "unique_id" parameter
     * embedded in the servlet request. If the unique_id is not
     * defined, then the method returns a virgin entry.
     */
    private Record getEntry() throws IOException
    {
        try
        {
            Integer recordID = new Integer(getParameter("unique_id"));
            return getData().get(recordID);
        }
        catch (Exception e)
        {
            return newEntry();
        }
    }


    /**
     * returns a new, virgin instance of a rolodex entry.
     */
    private Record newEntry() throws IOException
    {
        DataRegistry reg = DataRegistryImpl.getDefaultRegistry();
        return reg.getExemplar("rolodex_entry");
    }
}

