/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.data;

/**
 * An AntiRecord is to a regular record like Anti-matter is
 * to matter. A record's antirecord cancels it out.
 * This is really a trick so that we can treat a log with
 * deletions in it as a stream of records (the deletion is itself
 * a record.)
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</A>
 */
public class AntiRecord extends DefaultRecord
{
    Record nestedRec;

    public AntiRecord(Record nestedRec)
    {
        this.nestedRec = nestedRec;
        setMetadata(nestedRec.getMetadata());
    }

    public String toString()
    {
        return DefaultRecordInputStream.DELETION_MARKER
               + "\n"
               + nestedRec.toString();
    }

    public boolean isImmutable()
    {
        return nestedRec.isImmutable();
    }

    public Object getPrimaryKey()
    {
        return nestedRec.getPrimaryKey();
    }

    // This is basically meaningless in this context.
    void freeze() {}
}
