/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.data;

import java.util.Enumeration;

/**
 * A metadata object that describes what is in a record.
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</a>
 */

public interface RecordDescriptor extends java.io.Serializable
{
    /**
     * @return a string representing the type of record this is.
     */
    public String getType();

    /**
     * @return an integer representing the version number
     */
    public int getVersion();


   /**
    * @return a java.util.Enumeration of the
    * descriptors of the fields that comprise
    * this record.
    */
   Enumeration getFieldDescriptors();

   /**
    * @return the FieldDescriptor based
    * on name, or null if none is found.
    */
   FieldDescriptor getField(String name);

   /**
    * @return the FieldDescriptor based
    * on ordinal index.
    */
   FieldDescriptor getField(int i);

   /**
    * @return the FieldDescriptor
    * for the primary key.
    */
   FieldDescriptor getPrimaryKeyField();

   /**
    * @return the ordinal index of the field
    * of the given name, -1 if there is no
    * field of that name.
    */
   int indexOf(String name);

   /**
    * The number of fields in the record.
    */
   int size();

   /**
    * @return a custom class that is used to
    * "wrap" this record for presentation.
    * I anticipate that many (if not most)
    * implementations will simply return null
    * here and a default wrapper class will
    * be used.
    */
   Class getPresentationWrapperClass();

   /**
    * @return the Java class that corresponds
    * to this record type.
    */
   Class getRecordClass();

   /**
    * @return a Record that corresponds to
    * this record descriptor in its
    * default uninitialized state.
    */
   Record newRecord();


   /**
    * @return the name of the table that this
    * record type is stored in in a backing RDBMS.
    * Under most circumstances, this will
    * return the same as getType(), since there is
    * little reason not to name the table the same as
    * the underlying type.
    */
   String getTableName();
}
