/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.data;

import java.io.*;
import java.util.*;
import java.text.*;
import com.niggle.util.*;

/**
 * An implementation of com.niggle.data.Session that
 * leverages the functionality of the DefaultRecord implementation
 * @see com.niggle.data.inmemory.SimpleSessionManager
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</a>
 */
public class SessionImpl extends DefaultRecord implements Session
{
    static final String LIFETIME="lifetime";
    static final String USERNAME = "username";
    static final String KEY = "key";

    public SessionImpl()
    {
    }

    public String getUsername()
    {
        return (String) get(USERNAME);
    }

    public String getUniqueKey()
    {
        return (String) getPrimaryKey();
    }

    /**
     *  Checks the validity of the session.  If the owner matches the
     *  session's owner, and <code>expirationTime + lifeTime <
     *  currentTime</code>, returns <code>true</code>; otherwise, returns
     *  <code>false</code>.
     *
     *  @param owner the supposed owner of the session.
     *  @return the validity of the session.
     */
    public boolean isValidSession(String username)
    {
        return getUsername().equals(username) && !isStale();
    }

    /**
     * Is this session stale (e.g. expired through inactivity
     * or an explicit logout)
     * @return true if stale.
     */
    public boolean isStale()
    {
        if (super.isStale())
            return true;
        int lifetime = ((Integer) get(LIFETIME)).intValue() *1000;
        long expirationTime = getLastModified().getTime() + lifetime;
        boolean result = expirationTime < System.currentTimeMillis();
        if (result)
            markStale();
        return result;
    }
}
