/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.data.jdbc;

import java.io.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import com.niggle.data.*;
import com.niggle.util.*;

/**
 * A holder for some JDBC/SQL-related convenience routines
 */

abstract public class SQLUtil
{
    // REVISIT: Is this format the SQL standard? Will they all understand this? (JR)
    static DateFormat MYSQL_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


    static String buildInsertQuery(Record rec)
    {
        StringBuffer buf = new StringBuffer("INSERT INTO ");
        RecordDescriptor metadata = rec.getMetadata();
        buf.append(metadata.getTableName());
        buf.append("(");
        boolean first = true;
        for (Enumeration e = metadata.getFieldDescriptors(); e.hasMoreElements();)
        {
            FieldDescriptor fd = (FieldDescriptor) e.nextElement();
            if (!first)
            {
                buf.append(", ");
            }
            buf.append(fd.getName());
            first = false;
        }
        buf.append(") VALUES (");
        first = true;
        for (Enumeration e = metadata.getFieldDescriptors(); e.hasMoreElements();)
        {
            FieldDescriptor fd = (FieldDescriptor) e.nextElement();
            if (!first)
            {
                buf.append(", ");
            }
            buf.append(quoteValue(rec.get(fd)));
            first = false;
        }
        buf.append(")");
        return buf.toString();
    }

    static String buildUpdateQuery(Record rec)
    {
        StringBuffer buf = new StringBuffer("UPDATE ");
        RecordDescriptor metadata = rec.getMetadata();
        FieldDescriptor primaryKeyField = metadata.getPrimaryKeyField();
        buf.append(metadata.getTableName());
        boolean first = true;
        for (Enumeration e = metadata.getFieldDescriptors(); e.hasMoreElements();)
        {
            FieldDescriptor fd = (FieldDescriptor) e.nextElement();
            if (fd == primaryKeyField)
               continue;
            if (first)
               buf.append(" SET ");
            else
               buf.append(",");
            buf.append(fd.getName());
            buf.append("=");
            buf.append(quoteValue(rec.get(fd)));
            first = false;
        }
        buf.append(" WHERE ");
        buf.append(primaryKeyField.getName());
        buf.append("=");
        buf.append(quoteValue(rec.getPrimaryKey()));
        return buf.toString();
    }


    static String quoteValue(Object o)
    {
        if (o instanceof java.util.Date)
        {
            o = MYSQL_DATE_FORMAT.format((java.util.Date) o);
        }
        if (o instanceof String)
           return "'" + o + "'";
        else
            return o.toString();
    }

    static public void fillRecordFromResultSet(Record rec, ResultSet rs) throws SQLException
    {
        for (Enumeration enum = rec.getMetadata().getFieldDescriptors(); enum.hasMoreElements();)
        {
            FieldDescriptor field = (FieldDescriptor) enum.nextElement();
            // REVISIT: The following line is likely insufficient
            // and we may have to be fancier for the general case. (JR)
            Object val = rs.getObject(field.getName());
            rec.set(field, val);
        }
    }
}
