/*
 * Copyright (c) 2000 Reminders On-Line, Inc. (ROL)
 * All Rights Reserved. Unpublished.
 * This software is proprietary and confidential to ROL and may not be
 * duplicated, disclosed to third parties or used for any purpose not
 * expressly authorized by ROL.
 */
package com.niggle.data.jndi;

import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.spi.DirObjectFactory;
import com.niggle.data.*;
import com.niggle.util.*;

public class JNDIRecordFactory implements DirObjectFactory
{
    public JNDIRecordFactory() {}

    public Object getObjectInstance(Object obj, Name name, Context ctx,
                                    Hashtable env, Attributes inAttrs)
        throws Exception
    {
        Object result = null;

        // Short-circuit if we don't have any attributes.
        if (inAttrs == null)
            return result;

	if (obj instanceof DirContext)
        {
	    try {
                // Get our exemplar.
                Attribute attr = inAttrs.get(Record.TYPE_KEY);
                if (attr == null)
                    throw new AssertionFailedException("No type data.");
                String fullType = (String) attr.get();
                int i = fullType.indexOf('.');
                String type = fullType.substring(0, i);
                int version = -1;
                try {
                    version = Integer.parseInt(fullType.substring(++i));
                }
                catch (NumberFormatException e) {
                    throw new AssertionFailedException("Expected to extract the version from the full type: " + fullType);
                }
                DataRegistry registry = DataRegistryImpl.getDefaultRegistry();
                Record rec = registry.getExemplar(type, version);

                // Create our map of data values.
                Map table = new HashMap();
                Enumeration fields = rec.getMetadata().getFieldDescriptors();
                while (fields.hasMoreElements())
                {
                    FieldDescriptor field = (FieldDescriptor) fields.nextElement();
                    String fieldName = field.getName();
                    attr = inAttrs.get(fieldName);
                    if (attr != null)
                    {
                        String value = (String) attr.get();
                        table.put(fieldName, value);
                    }
                }

                // Fill it in.
                ((DefaultRecord) rec).setValuesFromTable(table);

                // Return our object.
                result = rec;
	    }
            catch (Exception e) {
		System.err.println(e);
		e.printStackTrace();
                throw e;
	    }
	}

	return result;
    }

    public Object getObjectInstance(Object obj, Name name, Context ctx,
                                    Hashtable env)
        throws Exception
    {
        return getObjectInstance(obj, name, ctx, env, null);
    }
}
