/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.data.metadata;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import com.niggle.util.Util;
import java.io.InputStream;
import java.io.File;

/**
 * Resolve entities from the classpath. This allows us to
 * jar up an xml document with more than one file comprising
 * its "physical structure".
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</a>
 */
public class ClasspathEntityResolver implements EntityResolver
{
    private ClassLoader cl;

    public ClasspathEntityResolver(ClassLoader cl)
    {
        this.cl = cl;
    }

    public ClasspathEntityResolver() {}

    /**
     * Tries to resolve an entity relative to the classloader's
     * search path. It first tries relative to the top-level document's
     * location and then tries relative to the top-level location.
     */
    public InputSource resolveEntity(String publicID, String systemID)
    {
        InputSource result = null;
        if (systemID.toLowerCase().startsWith("classpath:"))
            systemID = systemID.substring(10);
        else if (systemID.startsWith("file:/"))
            systemID = systemID.substring(6);
        while (systemID.startsWith("/"))
            systemID = systemID.substring(1);
        InputStream is = Util.getInputStream(systemID, cl);
        if (is != null)
        {
            result = new InputSource(is);
            result.setPublicId(publicID);
            result.setSystemId(systemID);
        }
        return result;
    }
}

