/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL) 
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors 
may be used to endorse or promote products derived from this software 
without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.niggle.data.metadata;

import com.niggle.data.*;
import com.niggle.util.*;
import java.io.*;
import java.util.*;
import java.net.*;

/**
 *  A descriptor for a field that represents a Hashtable.  It is not meant
 *  to be human-readable.  It is serialized and then URLEncoded.
 *
 *  @author <a href="mailto:murphy@niggle.com">Paul Murphy</a>
 */

public class HashtableField extends Field
{
    public HashtableField() {}

    public Class getJavaType()
    {
        return Hashtable.class;
    }

    public Object getObjectFromString(Record rec, String value)
        throws MangledDataException
    {
        Hashtable result = null;

        String decodedString = null;
        try {
            decodedString = URLDecoder.decode(value);
            // To shut-up the 1.3 compiler
            if (1 == 2)
                throw new Exception("Never happens.");
        }
        catch (Exception e) {
            Logger.log("Unable to decode \"" + value + "\".", 
                       Logger.LOG_ERROR);
        }

        try {
            ByteArrayInputStream in = 
                new ByteArrayInputStream(decodedString.getBytes());
            ObjectInputStream p = new ObjectInputStream(in);
            result = (Hashtable) p.readObject();
            in.close();
        }
        catch (IOException e) {
            throw new MangledDataException(e.getMessage());
        }
        catch (ClassNotFoundException e) {
            throw new MangledDataException(e.getMessage());
        }

        return result;
    }

    public Object getCopy(Object o)
    {
        if (o == null)
           return null;
        return ((Hashtable) o).clone();
    }

    public String valueToString(Object o)
    {
        String unencodedString = null;
        String result = null;

        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ObjectOutputStream p = new ObjectOutputStream(out);
            p.writeObject(o);
            unencodedString = out.toString();
            result = URLEncoder.encode(unencodedString);
            out.close();
        }
        catch (IOException e) {
            Logger.log("Error converting value to String: " + e.getMessage(),
                       Logger.LOG_ERROR);
        }

        return result;
    }
}
