/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL) 
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors 
may be used to endorse or promote products derived from this software 
without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.niggle.templates;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

/**
 *  An abstract encapsulation of a page template.
 * Currently, our implementation uses the Freemarker library.
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</a>
 */
public interface Page
{
    /**
     * Prepare the cooked output, filling in the parameters
     * with data values, and send it to the end user
     * via the servlet's response stream.  NB: the template
     * extension should determine the content type to generate.
     * <p>
     * These should be supported:
     * <ul>
     * <li>html, htm (default) => text/html</li>
     * <li>txt => text/ascii</li>
     * <li>xmml => text/xml</li>
     * </ul>
     *
     * @param response handle to where the output should go.
     * @exception IOException
     */
    void outputPage(HttpServletResponse response, boolean isStatic) throws IOException;

    /**
     * Expose internal data, publishing it to make it
     * available to the page template.
     *
     * @param key    name of the field in the template
     * @param item   source of the data to populate the template
     */
    void expose(String key, Object item);

    /**
     * Expose internal data, publishing it to make it
     * available to a Freemarker Template.
     *
     * @param key    name of the field in the template
     * @param b      sets to boolean value according to: b ? "true" : ""
     */
    void expose(String key, boolean b);
}
