/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.templates.freemarkerimpl;

import java.util.*;
import com.niggle.data.*;
import com.niggle.servlet.ServletInteraction;
import freemarker.template.*;

/**
 * A "lazy" list implementation of the freemarker
 * TemplateListModel interface
 * for exposing a collection of records at one go.
 * @author <a href="jrevusky@bigfoot.com">Jonathan Revusky</a>
 */

public class LazyListTemplate
extends FreemarkerPresentationWrapper implements TemplateListModel
{
    private int index;
    private List list;

    /**
     * @param list, a List of the records. A null will be
     * massaged into an empty list.
     */
    public LazyListTemplate(List list, ServletInteraction interaction, String contentType)
    {
        if (list == null)
           list = Collections.EMPTY_LIST;
        this.list = list;
        this.interaction = interaction;
        this.locale = interaction.getLocale();
        this.contentType = contentType;
    }

    public boolean isEmpty()
    {
        return list.isEmpty();
    }

    public void rewind()
    {
        index = 0;
    }

    public boolean isRewound()
    {
        return index == 0;
    }

    public TemplateModel next() throws TemplateModelException
    {
        if (!hasNext())
            throw new TemplateModelException();
        return this.get(index++);
    }

    public boolean hasNext()
    {
        return index < list.size();
    }

    public TemplateModel listSize() throws TemplateModelException
    {
        return new SmartScalar(new Integer(list.size()).toString());
    }

    public TemplateModel get(int i) throws TemplateModelException
    {
        if(i>=list.size())
            throw new TemplateModelException("IndexOutOfBounds(" + i + ")");
        Object o = list.get(i);
        TemplateModel result = createWrapper(o);
        if (result != o)
           list.set(i, result);
        return result;
    }
}
