/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.templates.freemarkerimpl;

import freemarker.template.*;
import com.niggle.data.*;
import com.niggle.servlet.ServletInteraction;
import com.niggle.data.metadata.*;
import com.niggle.util.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * A freemarker template model specifically designed
 * to "wrap" a niggle data record.
 * @author <a href="mailto:jrevusky@bigfoot.com">Jonathan Revusky</a>
 */
public class RecordTemplate extends LazyHashModel
{
    protected Record rec;

    public void setRecord(Record rec)
    {
        this.rec = rec;
    }

    public boolean isEmpty() throws TemplateModelException
    {
        return rec == null && super.isEmpty();
    }

    public TemplateModel get(String key) throws TemplateModelException
    {
        TemplateModel result = super.get(key);
        if (result != null)
           return result;
        if (rec!=null)
        {
            if (key.equalsIgnoreCase(Record.TYPE_KEY))
            {
                result = createWrapper(rec.getType());
            }
            else
            {
                FieldDescriptor field = rec.getMetadata().getField(key);
                if (field != null)
                   result = getWrappedFieldValue(field);
            }
        }
        if (result == null)
        {
            result = reflectionCheck(key);
        }
        put(key, result);
        return result;
    }


    /**
     * wrap the field value as a Freemarker TemplateModel
     */
    protected TemplateModel getWrappedFieldValue(FieldDescriptor field)
    {
        Object fieldValue = rec.get(field);
        if (field instanceof EnumField)
        {
            EnumField ef = (EnumField) field;
            String[] possibleValues = ef.getPossibleValues();
            EnumTemplateModel etm = new EnumTemplateModel(possibleValues, locale);
            if (fieldValue != null)
               etm.setSelection(fieldValue.toString());
            return etm;
        }
        return createWrapper(fieldValue);
    }


    protected TemplateModel reflectionCheck(String key)
    {
        TemplateModel result = null;
        if (!Util.hasContent(key))
           return null;
        String methodName = "get" +
               Character.toUpperCase(key.charAt(0))
               + key.substring(1).toLowerCase();
        String methodLookupName = rec.getType() + rec.getVersion() + methodName;
        Method m = null;
        try {
            if (methodLookup.containsKey(methodLookupName))
               m = (Method) methodLookup.get(methodLookupName);
            else
               m = rec.getClass().getMethod(methodName, EMPTY_CLASS_ARRAY);
            if (m != null)
               result = createWrapper(m.invoke(rec, EMPTY_CLASS_ARRAY));
        }
        catch (Exception e) {
        }
        methodLookup.put(methodLookupName, m);
        return result;
    }

    static final Class[] EMPTY_CLASS_ARRAY = new Class[0];
    static Map methodLookup = Collections.synchronizedMap(new HashMap());
}
