/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.niggle.util;

import java.io.*;

/**
 * This class provides logging to a file. Entries are appended to the end of the file.
 * @author Nicholas Christopher <nwc@ronin.nu>
 */
public class FileLog extends Object
  implements Log
{
    public static final String LINE_SEPARATOR = System.getProperty("line.separator");
    private String filename;

  /**
   * Intstantiant an instance for a given file. Determine system line separator
   * or use \n if all else fails.
   */
    public FileLog(String filename)
    {
      setFilename(filename);
    }

  /**
   * Set the value of the file.
   */
    final public void setFilename(String filename)
    {
        this.filename = filename;
    }

    final public String getFilename()
    {
        return this.filename;
    }


  /**
   * Append the o.toString() followed by a newline to the file.
   */
    public void println(Object o)
    {
        try {
            FileWriter fw = new FileWriter(filename, true);
            fw.write(o.toString());
            fw.write(LINE_SEPARATOR);
            fw.close();
        }
        catch(Exception e) {
            e.printStackTrace();
            System.err.println("Logging to file failed: " + e);
        }
    }
}

